pub struct IndexMeta {
pub index_settings: IndexSettings,
pub segments: Vec<SegmentMeta>,
pub schema: Schema,
pub opstamp: Opstamp,
pub payload: Option<String>,
}Expand description
Meta information about the Index.
This object is serialized on disk in the meta.json file.
It keeps information about
- the searchable segments,
- the index
docstamp - the schema
Fields
index_settings: IndexSettingsIndexSettings to configure index options.
segments: Vec<SegmentMeta>List of SegmentMeta information associated with each finalized segment of the index.
schema: SchemaIndex Schema
opstamp: OpstampOpstamp associated with the last commit operation.
payload: Option<String>Payload associated with the last commit.
Upon commit, clients can optionally add a small String payload to their commit
to help identify this commit.
This payload is entirely unused by tantivy.
Implementations