pub struct BoostQuery { /* private fields */ }Expand description
BoostQuery is a wrapper over a query used to boost its score.
The document set matched by the BoostQuery is strictly the same as the underlying query.
The score of each document, is the score of the underlying query multiplied by the boost
factor.
Implementations
Sourceimpl BoostQuery
impl BoostQuery