TantivyDocument

Struct TantivyDocument 

pub struct TantivyDocument {
    pub node_data: Vec<u8>,
    /* private fields */
}
The default document in tantivy. It encodes data in a compact form.

Fields

node_data: Vec<u8>

node_data is a vec of bytes, where each value is serialized into bytes and stored. It includes all the data of the document and also metadata like where the nodes are located in an object or array.

Implementations

Source

impl CompactDoc

pub fn with_capacity(bytes: usize) -> CompactDoc

Creates a new, empty document object The reserved capacity is for the total serialized data

pub fn new() -> CompactDoc

Creates a new, empty document object

pub fn shrink_to_fit(&mut self)

Skrinks the capacity of the document to fit the data

pub fn len(&self) -> usize

Returns the length of the document.

pub fn add_facet<F>(&mut self, field: Field, path: F)
where Facet: From<F>,

Adding a facet to the document.

pub fn add_text<S: AsRef<str>>(&mut self, field: Field, text: S)

Add a text field.

pub fn add_pre_tokenized_text( &mut self, field: Field, pre_tokenized_text: PreTokenizedString, )

Add a pre-tokenized text field.

pub fn add_u64(&mut self, field: Field, value: u64)

Add a u64 field

pub fn add_ip_addr(&mut self, field: Field, value: Ipv6Addr)

Add a IP address field. Internally only Ipv6Addr is used.

pub fn add_i64(&mut self, field: Field, value: i64)

Add a i64 field

pub fn add_f64(&mut self, field: Field, value: f64)

Add a f64 field

pub fn add_bool(&mut self, field: Field, value: bool)

Add a bool field

pub fn add_date(&mut self, field: Field, value: DateTime)

Add a date field with unspecified time zone offset

pub fn add_bytes(&mut self, field: Field, value: &[u8])

Add a bytes field

pub fn add_object(&mut self, field: Field, object: BTreeMap<String, OwnedValue>)

Add a dynamic object field

pub fn add_field_value<'a, V: Value<'a>>(&mut self, field: Field, value: V)

Add a (field, value) to the document.

OwnedValue implements Value, which should be easiest to use, but is not the most performant.

pub fn add_leaf_field_value<'a, T: Into<ReferenceValueLeaf<'a>>>( &mut self, field: Field, typed_val: T, )

Add a (field, leaf value) to the document. Leaf values don’t have nested values.

pub fn field_values(&self) -> impl Iterator<Item = (Field, CompactDocValue<'_>)>

field_values accessor

pub fn get_all( &self, field: Field, ) -> impl Iterator<Item = CompactDocValue<'_>> + '_

Returns all of the ReferenceValues associated the given field

pub fn get_first(&self, field: Field) -> Option<CompactDocValue<'_>>

Returns the first ReferenceValue associated the given field

pub fn convert_named_doc( schema: &Schema, named_doc: NamedFieldDocument, ) -> Result<Self, DocParsingError>

Create document from a named doc.

pub fn parse_json( schema: &Schema, doc_json: &str, ) -> Result<Self, DocParsingError>

Build a document object from a json-object.

pub fn from_json_object( schema: &Schema, json_obj: Map<String, Value>, ) -> Result<Self, DocParsingError>

Build a document object from a json-object.