Module containing the different query implementations.
Structs
- AllQuery
- Query that matches all of the documents.
- AllScorer
- Scorer associated with the
AllQueryquery. - AllWeight
- Weight associated with the
AllQueryquery. - Automaton
Weight - A weight struct for Fuzzy Term and Regex Queries
- BitSet
DocSet - A
BitSetDocSetmakes it possible to iterate through a bitset as if it was aDocSet. - Bm25
Weight - A struct used for computing BM25 scores.
- Boolean
Query - The boolean query returns a set of documents that matches the Boolean combination of constituent subqueries.
- Boolean
Weight - Weight associated to the
BoolQuery. - Boost
Query BoostQueryis a wrapper over a query used to boost its score.- Boost
Weight - Weight associated to the BoostQuery.
- Bounds
Range - Buffered
Union Scorer - Creates a
DocSetthat iterate through the union of two or moreDocSets. - Const
Score Query ConstScoreQueryis a wrapper over a query to provide a constant score. It can avoid unnecessary score computation on the wrapped query.- Const
Scorer - Wraps a
DocSetand simply returns a constantScorer. TheConstScoreris useful if you have aDocSetwhere you needed a scorer. - Disjunction
MaxCombiner - Take max score of different scorers
and optionally sum it with other matches multiplied by
tie_breaker - Disjunction
MaxQuery - The disjunction max query returns documents matching one or more wrapped queries, called query clauses or clauses.
- Empty
Query EmptyQueryis a dummyQueryin which no document matches.- Empty
Scorer EmptyScoreris a dummyScorerin which no document matches.- Empty
Weight EmptyWeightis a dummyWeightin which no document matches.- Exclude
- Filters a given
DocSetby removing the docs from a givenDocSet. - Exists
Query - Query that matches all documents with a non-null value in the specified field.
- Explanation
- Object describing the score of a given document. It is organized in trees.
- Fast
Field Range Query FastFieldRangeQueryis the same as [RangeQuery] but only uses the fast field- Fast
Field Range Weight FastFieldRangeWeightuses the fast field to execute range queries.- Fuzzy
Term Query - A Fuzzy Query matches all of the documents containing a specific term that is within Levenshtein distance
- Intersection
- Creates a
DocSetthat iterate through the intersection of two or moreDocSets. - Inverted
Index Range Query InvertedIndexRangeQueryis the same as RangeQuery but only uses the inverted index- Inverted
Index Range Weight - Range weight on the inverted index
- More
Like This Query - A query that matches all of the documents similar to a document or a set of field values provided.
- More
Like This Query Builder - The builder for more-like-this query
- Phrase
Prefix Query PhrasePrefixQuerymatches a specific sequence of words followed by term of which only a prefix is known.- Phrase
Query PhraseQuerymatches a specific sequence of words.- Query
Parser - Tantivy’s Query parser
- Range
Query RangeQuerymatches all documents that have at least one term within a defined range.- Regex
Phrase Query RegexPhraseQuerymatches a specific sequence of regex queries.- Regex
Query - A Regex Query matches all of the documents containing a specific term that matches a regex pattern.
- Required
Optional Scorer - Given a required scorer and an optional scorer matches all document from the required scorer and complements the score using the optional scorer.
- SumCombiner
- Sums the score of different scorers.
- Term
Query - A Term query matches all of the documents containing a specific term.
- Term
SetQuery - A Term Set Query matches all of the documents containing any of the Term provided
Enums
- Enable
Scoring - Argument used in
Query::weight(..) - Occur
- Defines whether a term in a query must be present, should be present or must not be present.
- Query
Parser Error - Possible error that may happen when parsing a query.
Traits
- Bm25
Statistics Provider - An interface to compute the statistics needed in BM25 scoring.
- Query
- The
Querytrait defines a set of documents and a scoring method for those documents. - Query
Clone - Implements
box_clone. - Score
Combiner - The
ScoreCombinertrait defines how to compute an overall score given a list of scores. - Scorer
- Scored set of documents matching a query within a specific segment.
- Weight
- A Weight is the specialization of a
Queryfor a given set of segments.
Functions
- intersect_
scorers - Returns the intersection scorer.
- wildcard_
query_ to_ regex_ str - Transform a wildcard query to a regex string.