pub struct TantivyDocument {
pub node_data: Vec<u8>,
/* private fields */
}Expand description
The default document in tantivy. It encodes data in a compact form.
Fields
node_data: Vec<u8>node_data is a vec of bytes, where each value is serialized into bytes and stored. It
includes all the data of the document and also metadata like where the nodes are located
in an object or array.
Implementations
Sourceimpl CompactDoc
impl CompactDoc
Sourcepub fn with_capacity(bytes: usize) -> CompactDoc
pub fn with_capacity(bytes: usize) -> CompactDoc
Creates a new, empty document object The reserved capacity is for the total serialized data
Sourcepub fn new() -> CompactDoc
pub fn new() -> CompactDoc
Creates a new, empty document object
Sourcepub fn shrink_to_fit(&mut self)
pub fn shrink_to_fit(&mut self)
Skrinks the capacity of the document to fit the data
Sourcepub fn add_pre_tokenized_text(
&mut self,
field: Field,
pre_tokenized_text: PreTokenizedString,
)
pub fn add_pre_tokenized_text( &mut self, field: Field, pre_tokenized_text: PreTokenizedString, )
Add a pre-tokenized text field.
Sourcepub fn add_ip_addr(&mut self, field: Field, value: Ipv6Addr)
pub fn add_ip_addr(&mut self, field: Field, value: Ipv6Addr)
Add a IP address field. Internally only Ipv6Addr is used.
Sourcepub fn add_date(&mut self, field: Field, value: DateTime)
pub fn add_date(&mut self, field: Field, value: DateTime)
Add a date field with unspecified time zone offset
Sourcepub fn add_object(&mut self, field: Field, object: BTreeMap<String, OwnedValue>)
pub fn add_object(&mut self, field: Field, object: BTreeMap<String, OwnedValue>)
Add a dynamic object field
Sourcepub fn add_field_value<'a, V: Value<'a>>(&mut self, field: Field, value: V)
pub fn add_field_value<'a, V: Value<'a>>(&mut self, field: Field, value: V)
Add a (field, value) to the document.
OwnedValue implements Value, which should be easiest to use, but is not the most
performant.
Sourcepub fn add_leaf_field_value<'a, T: Into<ReferenceValueLeaf<'a>>>(
&mut self,
field: Field,
typed_val: T,
)
pub fn add_leaf_field_value<'a, T: Into<ReferenceValueLeaf<'a>>>( &mut self, field: Field, typed_val: T, )
Add a (field, leaf value) to the document. Leaf values don’t have nested values.
Sourcepub fn field_values(&self) -> impl Iterator<Item = (Field, CompactDocValue<'_>)>
pub fn field_values(&self) -> impl Iterator<Item = (Field, CompactDocValue<'_>)>
field_values accessor
Sourcepub fn get_all(
&self,
field: Field,
) -> impl Iterator<Item = CompactDocValue<'_>> + '_
pub fn get_all( &self, field: Field, ) -> impl Iterator<Item = CompactDocValue<'_>> + '_
Returns all of the ReferenceValues associated the given field
Sourcepub fn get_first(&self, field: Field) -> Option<CompactDocValue<'_>>
pub fn get_first(&self, field: Field) -> Option<CompactDocValue<'_>>
Returns the first ReferenceValue associated the given field
Sourcepub fn convert_named_doc(
schema: &Schema,
named_doc: NamedFieldDocument,
) -> Result<Self, DocParsingError>
pub fn convert_named_doc( schema: &Schema, named_doc: NamedFieldDocument, ) -> Result<Self, DocParsingError>
Create document from a named doc.
Sourcepub fn parse_json(
schema: &Schema,
doc_json: &str,
) -> Result<Self, DocParsingError>
pub fn parse_json( schema: &Schema, doc_json: &str, ) -> Result<Self, DocParsingError>
Build a document object from a json-object.
Sourcepub fn from_json_object(
schema: &Schema,
json_obj: Map<String, Value>,
) -> Result<Self, DocParsingError>
pub fn from_json_object( schema: &Schema, json_obj: Map<String, Value>, ) -> Result<Self, DocParsingError>
Build a document object from a json-object.