pub struct TrackedObject<T> { /* private fields */ }Expand description
Your tracked object.
A tracked object contains reference counting logic and an
Arc to your object. It is cloneable but calling clone will
not clone your internal object.
Your object cannot be mutated. You can borrow it using
the Deref interface.
Implementations
Sourceimpl<T> TrackedObject<T>
impl<T> TrackedObject<T>
Sourcepub fn map<F>(&self, f: F) -> TrackedObject<T>
pub fn map<F>(&self, f: F) -> TrackedObject<T>
Creates a new object from an existing one.
The new object will be registered in your original object’s inventory.
use census::{Inventory, TrackedObject};
let inventory = Inventory::new();
let seven = inventory.track(7);
let fourteen = seven.map(|i| i * 2);
assert_eq!(*fourteen, 14);
let living_instances = inventory.list();
assert_eq!(living_instances.len(), 2);Trait Implementations
Sourceimpl<T> AsRef<T> for TrackedObject<T>
impl<T> AsRef<T> for TrackedObject<T>
Sourceimpl<T> Borrow<T> for TrackedObject<T>
impl<T> Borrow<T> for TrackedObject<T>
Sourceimpl<T> Clone for TrackedObject<T>where
T: Clone,
impl<T> Clone for TrackedObject<T>where
T: Clone,
Sourcefn clone(&self) -> TrackedObject<T>
fn clone(&self) -> TrackedObject<T>
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl<T> Debug for TrackedObject<T>where
T: Debug,
impl<T> Debug for TrackedObject<T>where
T: Debug,
Sourceimpl<T> Deref for TrackedObject<T>
impl<T> Deref for TrackedObject<T>
Sourceimpl From<TrackedObject<InnerSegmentMeta>> for SegmentMeta
impl From<TrackedObject<InnerSegmentMeta>> for SegmentMeta
Sourcefn from(tracked: TrackedObject<InnerSegmentMeta>) -> SegmentMeta
fn from(tracked: TrackedObject<InnerSegmentMeta>) -> SegmentMeta
Converts to this type from the input type.